import javax.swing.table.AbstractTableModel;

class SignalTableModel extends AbstractTableModel {
	private String[] columnNames = { "Signal", "Slot" };
	private XSignal[] signals = new XSignal[0];
	private String elementName = null;

	public void clear() {
		this.elementName = null;
		this.signals = new XSignal[0];
		fireTableStructureChanged();
	}

	public void update(String elementName, XSignal[] signals) {
		this.elementName = elementName;
		this.signals = signals;
		fireTableStructureChanged();
	}

	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return this.signals.length;
	}

	public String getColumnName(int col) {
		return columnNames[col];
	}

	public Object getValueAt(int row, int col) {
		return (col == 0 ?  signals[row].getName() : (signals[row].hasMapping() ? signals[row].getMappingName() : "None"));
	}

	public boolean isCellEditable(int row, int col) {
		return (col == 1);
	}

	public void setValueAt(Object value, int row, int col) {
		if (value.equals("New Slot Mapping...")) {
			String mapping_name = elementName + "_" + this.signals[row].getName();
			XSlotMapping res = new AddMappingDialog(mapping_name).showDialog();
			if (res != null) {
				Editor.addSlotMapping(res.getName(), res);
				this.signals[row].setMappingName(res.getName());
			} else {
				this.signals[row].setMappingName(null);
			}
		} else if (value.equals("None")) {
			this.signals[row].setMappingName(null);
		} else {
			this.signals[row].setMappingName(value.toString());
		}
		fireTableCellUpdated(row, col);
	}
}