import java.util.Map;
import java.util.LinkedHashMap;
import java.awt.Component;

abstract class XElement {
	protected Component component;
	protected Map<String, XProperty> properties;
	protected Map<String, XSignal> signals;

	public XElement() {
		this.properties = new LinkedHashMap<String, XProperty>();
		this.signals = new LinkedHashMap<String, XSignal>();
	}

	public XElement(String id) {
		this.properties = new LinkedHashMap<String, XProperty>();
		this.setProperty("id", new XPropertyID("id", "ID", id));
		this.signals = new LinkedHashMap<String, XSignal>();
	}

	public void setProperty(String name, XProperty property) {
		this.properties.put(name, property);
	}

	public XProperty getProperty(String name) {
		return this.properties.get(name);
	}

	public XProperty[] getProperties() {
		return this.properties.values().toArray(new XProperty[0]);
	}

	public void setSignal(String name, XSignal signal) {
		this.signals.put(name, signal);
	}

	public XSignal getSignal(String name) {
		return this.signals.get(name);
	}

	public XSignal[] getSignals() {
		return this.signals.values().toArray(new XSignal[0]);
	}

	public Component getComponent() {
		return this.component;
	}

	public String toString() {
		return this.properties.get("id").getValue().toString();
	}

	abstract void generateComponent();
	abstract void updateComponent();
	abstract boolean hasChildren();
}