import javax.swing.JTextField;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import javax.swing.SwingUtilities;
import java.awt.Cursor;
import java.awt.dnd.DropTarget;

public class XTextBox extends XElementLeaf {
	public XTextBox() {
		this.setSignal("TextChanged", new XSignal("TextChanged"));
	}

	public XTextBox(String id, String text, int width, int height, int top, int left) {
		super(id);
		this.setProperty("left", new XPropertyInteger("left", "Left", left));
		this.setProperty("top", new XPropertyInteger("top", "Top", top));
		this.setProperty("width", new XPropertyInteger("width", "Width", width));
		this.setProperty("height", new XPropertyInteger("height", "Height", height));
		this.setProperty("text", new XPropertyString("text", "Text", text));
		this.generateComponent();
	}

	public void generateComponent() {
		final JTextField textbox = new JTextField();
		textbox.setFocusable(false);
		textbox.setCursor(new Cursor(Cursor.MOVE_CURSOR));
		DropTarget d = new DropTarget();
		d.setActive(false);
		textbox.setDropTarget(d);
		this.component = (java.awt.Component)textbox;
		this.updateComponent();

		final Point offset = new Point(-1, -1);

		textbox.addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent evt) {
				if (offset.getX() == -1 && offset.getY() == -1) {
					offset.setLocation(MouseInfo.getPointerInfo().getLocation());
					SwingUtilities.convertPointFromScreen(offset, textbox);
				}
				move(MouseInfo.getPointerInfo().getLocation(), offset);
			}
		});

		textbox.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				offset.setLocation(-1, -1);
			}
		});
	}

	public void updateComponent() {
		JTextField textbox = ((JTextField)this.component);
		textbox.setText((String)this.getProperty("text").getValue());
		textbox.setLocation(
			(Integer)this.getProperty("left").getValue(),
			(Integer)this.getProperty("top").getValue()
		);
		textbox.setSize(new Dimension(
			(Integer)this.getProperty("width").getValue(),
			(Integer)this.getProperty("height").getValue()
		));
	}

	private void move(Point position, Point offset) {
		JTextField button = ((JTextField)this.component);
		SwingUtilities.convertPointFromScreen(position, Editor.getEditorWindow().getWindowVisualiserContentPane());
		position.translate((int)offset.getX() * -1, (int)offset.getY() * -1);
		position.setLocation((int)(Math.rint((double) position.getX() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE),
								(int)(Math.rint((double) position.getY() / PreferenceManager.GRID_SIZE) * PreferenceManager.GRID_SIZE));
		button.setLocation(position);
		this.getProperty("left").setValue((int)(position.getX()));
		this.getProperty("top").setValue((int)(position.getY()));
	}
}