import xml.etree.ElementTree as ET
import wx
from PIL import Image
from time import clock, time

class XElement(object):
	_properties = {}

	def __init__(self):
		self._properties = {}

	def GetPropertyValue(self, key):  # Returns the property value or None if property doesn't exist
		if (self._properties.has_key(key)):
			return self._properties[key]
		return None

	def SetPropertyValue(self, key, value):
		self._properties[key] = value

	def HasProperty(self, key):
		return self._properties.has_key(key)

class XElementBranch(XElement):
	def __init__(self):
		super(XElementBranch, self).__init__()

class XElementLeaf(XElement):
	def __init__(self):
		super(XElementLeaf, self).__init__()

class XWindow(XElementBranch):
	def __init__(self):
		super(XWindow, self).__init__()

	def GenerateComponent(self, parent=None):
		resizable = (self._properties["resizable"] == "True")
		closable = (self._properties["closable"] == "True")
		minimizable = (self._properties["minimizable"] == "True")
		maximizable = (self._properties["maximizable"] == "True")

		wstyle = wx.DEFAULT_FRAME_STYLE
		if (resizable == False):
			wstyle = wstyle & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX)
		if (closable == False):
			wstyle = wstyle & ~ (wx.CLOSE_BOX)
		if (minimizable == False):
			wstyle = wstyle & ~ (wx.MINIMIZE_BOX)
		if (maximizable == False):
			wstyle = wstyle & ~ (wx.MAXIMIZE_BOX)

		self._component = wx.Frame(None, wx.ID_ANY, self._properties["text"], size=(int(self._properties["width"]), int(self._properties["height"])), style=wstyle)
		difference = self._component.GetSize() - self._component.GetClientSize()
		self._component.SetSize(self._component.GetSize() + difference)
		self._panel = wx.Panel(self._component, wx.ID_ANY, pos=wx.DefaultPosition)

	def GetContentComponent(self):
		return self._panel

	def Show(self):
		self._component.Show(True)

class XGroupBox(XElementBranch):
	def __init__(self):
		super(XGroupBox, self).__init__()

	def GenerateComponent(self, window):
		self._component = wx.StaticBox(window, wx.ID_ANY, label=self._properties["text"], size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))
		self._panel = wx.Panel(window, wx.ID_ANY, size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))

	def GetContentComponent(self):
		return self._panel

class XPanel(XElementBranch):
	def __init__(self):
		super(XPanel, self).__init__()

	def GenerateComponent(self, window):
		self._component = wx.Panel(window, wx.ID_ANY, size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))

	def GetContentComponent(self):
		return self._component

class XButton(XElementLeaf):
	def __init__(self):
		super(XButton, self).__init__()
	
	def GenerateComponent(self, window):
		self._component = wx.Button(window, -1, label=self._properties["text"], size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))

class XTextBox(XElementLeaf):
	def __init__(self):
		super(XTextBox, self).__init__()

	def _UpdateText(self, event):
		self._properties["text"] = self._component.GetValue()

	def GenerateComponent(self, window):
		self._component = wx.TextCtrl(window, -1, value=str(self._properties["text"]), size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))
		self._component.Bind(wx.EVT_TEXT, self._UpdateText)

class XLabel(XElementLeaf):
	def __init__(self):
		super(XLabel, self).__init__()

	def GenerateComponent(self, window):
		self._component = wx.StaticText(window, -1, label=self._properties["text"], size=(int(self._properties["width"]), int(self._properties["height"])), pos=(int(self._properties["left"]), int(self._properties["top"])))

	def SetText(self, text):
		self._component.SetLabel(text)
		self._properties["text"] = text

	def GetText(self):
		return self._properties["text"]

class XImageBox(XElementLeaf):
	def __init__(self):
		super(XImageBox, self).__init__()

	def GenerateComponent(self, window):
		self._properties["imagesource"] = wx.Image(self._properties["imagesource"], wx.BITMAP_TYPE_ANY).ConvertToBitmap()
		self._component = wx.StaticBitmap(window, wx.ID_ANY, self._properties["imagesource"], (int(self._properties["left"]), int(self._properties["top"])), (int(self._properties["width"]), int(self._properties["height"])))

	def GetImage(self):
		img = wx.ImageFromBitmap(self._properties["imagesource"])
		myPilImage = Image.new( 'RGB', (img.GetWidth(), img.GetHeight()) )
		myPilImage.fromstring( img.GetData() )
		return myPilImage
		#return self._properties["imagesource"]

	def SetImage(self, image):
		myWxImage = wx.EmptyImage(image.size[0], image.size[1])
		myWxImage.SetData(image.convert('RGB').tostring())
		img = myWxImage.ConvertToBitmap()
		myWxImage = None
		self._component.SetBitmap(img)


signalmap = { 'Click': wx.EVT_BUTTON }

class Connection:
	_element = None
	_slotMapping = None
	_signalEvent = None

	def __init__(self, element, signal, slotMapping):
		self._element = element
		self._slotMapping = slotMapping
		self._signalEvent = signalmap[signal]

	def GetElement(self):
		return self._element

	def GetSignalEvent(self):
		return self._signalEvent

	def GetSlotMapping(self):
		return self._slotMapping

class SlotMapping:
	_slotName = None
	_bindings = None

	def __init__(self, slotName):
		self._slotName = slotName
		self._bindings = []

	def AddBinding(self, binding):
		self._bindings.append(binding)

	def GetSlotName(self):
		return self._slotName

	def GetBindings(self):
		return self._bindings

class SlotMappingBinding:
	_parameter = None
	_element = None
	_property = None

	def __init__(self, parameter, element, property):
		self._parameter = parameter
		self._element = element
		self._property = property

	def GetParameter(self):
		return self._parameter

	def GetElement(self):
		return self._element

	def GetProperty(self):
		return self._property

classmap = { 'XWindow': XWindow, 'XButton': XButton, 'XTextBox': XTextBox, 'XLabel': XLabel, 'XGroupBox': XGroupBox, 'XPanel': XPanel, 'XImageBox': XImageBox }

class GUI:
	_elements = {}
	_connections = []
	_slotMappings = {}
	_methods = {}
	_rootElementID = None
	_app = None

	def Load(self, filepath):
		start = clock()
		tree = ET.parse(filepath)
		root = tree.getroot()
		window = root.find('elements').find('XWindow')
		self._app = wx.App(False)
		self._ProcessGUI(window, None)

		slotMappings = list(root.find("slot_mappings"))
		for sm in slotMappings:
			slotMapping = SlotMapping(sm.find("slot").text)
			bindings = sm.find("bindings")
			if bindings != None:
				for sm_binding in list(bindings):
					slotMappingBinding = SlotMappingBinding(sm_binding.find("parameter").text, self._elements[sm_binding.find("element").text], sm_binding.find("property").text)
					slotMapping.AddBinding(slotMappingBinding)
			#self._slotMappings[sm.find("name").text] = slotMapping
			self._slotMappings[sm.attrib["name"]] = slotMapping

		conns = list(root.find("connections"))
		for con in conns:
			self._connections.append(Connection(self._elements[con.find("element").text], con.find("signal").text, self._slotMappings[con.find("slot_mapping").text]))

		end = clock()
		print "Executed in ", (end-start)


	def GetElementByID(self, id):
		if (self._elements.has_key(id)):
			return self._elements[id]
		return None


	def _ProcessGUI(self, element, parent):
		print "Found", element.tag, "with parent", parent

		component = classmap[element.tag]()

		for property in list(element):
			if (property.tag != "elements"):
				component.SetPropertyValue(property.tag, property.text)

		if (len(self._elements) == 0):
			component.GenerateComponent()
			self._rootElementID = element.attrib["id"]
		else:
			component.GenerateComponent(parent.GetContentComponent())

		self._elements[element.attrib["id"]] = component

		if element.find("elements") != None:
			for el in element.find("elements"):
				self._ProcessGUI(el, component)

	def Connect(self, methods):
		for method in methods:
			self._methods[method.__name__] = method
		for c in self._connections:
			c.GetElement()._component.Bind(c.GetSignalEvent(), lambda evt, temp=c.GetSlotMapping(): self._DispatchEvent(evt, temp))

	def _DispatchEvent(self, event, slotMapping):
		args = {}
		for binding in slotMapping.GetBindings():
			args[binding.GetParameter()] = binding.GetElement().GetPropertyValue(binding.GetProperty())
		result = self._methods[slotMapping.GetSlotName()](**args)
		print result

	def Display(self):
		self._elements[self._rootElementID].Show()
		#SetTopWindow(self._elements[self._rootElementID])
		self._app.MainLoop()

